/*
    siefs: a virtual filesystem for accessing Siemens mobiles
    Copyright (C) 2003  Dmitry Zakharov (dmitry-z@mail.ru)

    This program can be distributed under the terms of the GNU GPL.
    See the file COPYING.
*/

/* communication layer */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include "comm.h"

#define DEFBAUDRATE B19200

int comm_open(char *device) {

	int h;
	struct termios tio;
	
	h = open(device, O_RDWR | O_NOCTTY );
	if (h < 0) return -1;

	bzero(&tio, sizeof(tio));
	tio.c_cflag = DEFBAUDRATE | PARODD | HUPCL | CS8 | CLOCAL | CREAD;
	tio.c_iflag = IGNPAR | IGNBRK;
	tio.c_oflag = 0;
	tio.c_lflag = 0;

	tio.c_cc[VTIME]    = 10;
	tio.c_cc[VMIN]     = 0;

	tcflush(h, TCIOFLUSH);
	if (tcsetattr(h, TCSANOW, &tio) != 0)
		return -1;

	return h;

}

int comm_setspeed(int h, int speed) {

	struct termios tio;
	int i;	
	const struct {
		int speed;
		int value;
	}
	r[] = { { 2400, B2400 }, { 9600, B9600 }, { 19200, B19200 }, 
	      { 38400, B38400 }, { 57600, B57600 }, { 115200, B115200 },
	      { 230400, B230400 }, { 0, 0 } };

	for (i=0; r[i].speed != speed; i++)
		if (r[i].speed == 0) { errno = EINVAL; return -1; }

	tcgetattr(h, &tio);
	tio.c_cflag = r[i].value | PARODD | HUPCL | CS8 | CLOCAL | CREAD;
	if (tcsetattr(h, TCSANOW, &tio) != 0)
		return -1;

	return 0;
}

int comm_settimeout(int h, int timeout) {

	struct termios tio;

	tcgetattr(h, &tio);
	tio.c_cc[VTIME]    = timeout;
	if (tcsetattr(h, TCSANOW, &tio) != 0)
		return -1;

	return 0;
}
	
int comm_rx(int h, void *buf, int len) {

	int c, n=0;
	
	while (n < len){
		c = read(h, buf+n, 1);
		if (c < 0) return -1;
		if (c == 0) break;
		n += c;
	}

	return n;
}

int comm_tx(int h, void *buf, int len) {

	int c, n=0;
	
	while (n < len){
		c = write(h, buf+n, 1);
		if (c < 0) return -1;
		if (c == 0) break;
		n += c;
	}

	return n;
}

int comm_printf(int h, const char *fmt, ...) {

	char *buf;
	va_list ap;
	int n;

	buf = malloc(4096);
	va_start(ap, fmt);
	n = vsnprintf(buf, 4096, fmt, ap);
	va_end(ap);

	n = comm_tx(h, buf, n);
	free(buf);
	return n;
}	

int comm_getline(int h, char *buf, int size) {

	int c, n=0;

	do {
		c = comm_rx(h, buf+n, 1);
		if (c < 0) return -1;
		if (c == 0) break;
		n ++;
		if (buf[n-1] == '\n') break;
	} while (n < size);

	return n;
}
	
int comm_close(int h) {

	close(h);
}

